@@ -1,17 +1,23 @@
-import sets, random, sha, base64
+import random, base64
 from zope.interface import implements
 from twisted.internet import defer
 from twisted.python.util import InsensitiveDict
 from ldaptor import interfaces, attributeset, delta
 from ldaptor.protocols.ldap import distinguishedname, ldif, ldaperrors
 
+try:
+    from hashlib import sha1
+except ImportError:
+    from sha import sha as sha1
+
+
 def sshaDigest(passphrase, salt=None):
     if salt is None:
         salt = ''
         for i in range(8):
             salt += chr(random.randint(0, 255))
 
-    s = sha.sha()
+    s = sha1()
     s.update(passphrase)
     s.update(salt)
     encoded = base64.encodestring(s.digest()+salt).rstrip()
@@ -60,10 +66,32 @@ class BaseLDAPEntry(object):
         return self.has_key(key)
 
     def keys(self):
-        return self._attributes.keys()
+        a = []
+        if self.get('objectClass'):
+            a.append('objectClass')
+        l=list(self._attributes.keys())
+        l.sort()
+        for key in l:
+            if key.lower() != 'objectclass':
+                a.append(key)
+        return a
 
     def items(self):
-        return self._attributes.items()
+        a=[]
+        objectClasses = list(self.get('objectClass', []))
+        objectClasses.sort()
+        if objectClasses:
+            a.append(('objectClass', objectClasses))
+
+        l=list(self._attributes.items())
+        l.sort()
+        for key, values in l:
+            if key.lower() != 'objectclass':
+                vs = list(values)
+                vs.sort()
+                a.append((key, vs))
+
+        return a
 
     def __str__(self):
         a=[]
@@ -139,8 +167,8 @@ class BaseLDAPEntry(object):
 
         r = []
 
-        myKeys = sets.Set(self.keys())
-        otherKeys = sets.Set(other.keys())
+        myKeys = set(self.keys())
+        otherKeys = set(other.keys())
 
         addedKeys = list(otherKeys - myKeys)
         addedKeys.sort() # for reproducability only
