@@ -16,9 +16,9 @@
 
 """LDAP protocol server"""
 
-from ldaptor import interfaces
+from ldaptor import interfaces, delta
 from ldaptor.protocols import pureldap, pureber
-from ldaptor.protocols.ldap import ldaperrors, distinguishedname, ldaperrors
+from ldaptor.protocols.ldap import distinguishedname, ldaperrors
 
 from twisted.python import log
 from twisted.internet import protocol, defer
@@ -30,8 +30,8 @@ class BaseLDAPServer(protocol.Protocol):
     debug = False
 
     def __init__(self):
-	self.buffer = ''
-	self.connected = None
+        self.buffer = ''
+        self.connected = None
 
     berdecoder = pureldap.LDAPBERDecoderContext_TopLevel(
         inherit=pureldap.LDAPBERDecoderContext_LDAPMessage(
@@ -39,35 +39,35 @@ class BaseLDAPServer(protocol.Protocol):
         inherit=pureldap.LDAPBERDecoderContext(fallback=pureber.BERDecoderContext())))
 
     def dataReceived(self, recd):
-	self.buffer += recd
-	while 1:
-	    try:
-		o, bytes=pureber.berDecodeObject(self.berdecoder, self.buffer)
-	    except pureldap.BERExceptionInsufficientData:
-		o, bytes=None, 0
+        self.buffer += recd
+        while 1:
+            try:
+                o, bytes=pureber.berDecodeObject(self.berdecoder, self.buffer)
+            except pureber.BERExceptionInsufficientData: #TODO
+                o, bytes=None, 0
             self.buffer = self.buffer[bytes:]
-	    if o is None:
-		break
-	    self.handle(o)
+            if o is None:
+                break
+            self.handle(o)
 
     def connectionMade(self):
-	"""TCP connection has opened"""
-	self.connected = 1
+        """TCP connection has opened"""
+        self.connected = 1
 
     def connectionLost(self, reason=protocol.connectionDone):
-	"""Called when TCP connection has been lost"""
-	self.connected = 0
+        """Called when TCP connection has been lost"""
+        self.connected = 0
 
     def queue(self, id, op):
-	if not self.connected:
-	    raise LDAPServerConnectionLostException()
-	msg=pureldap.LDAPMessage(op, id=id)
+        if not self.connected:
+            raise LDAPServerConnectionLostException()
+        msg=pureldap.LDAPMessage(op, id=id)
         if self.debug:
-            log.debug('S->C %s' % repr(msg))
-	self.transport.write(str(msg))
+            log.msg('S->C %s' % repr(msg), debug=True)
+        self.transport.write(str(msg))
 
     def unsolicitedNotification(self, msg):
-	log.msg("Got unsolicited notification: %s" % repr(msg))
+        log.msg("Got unsolicited notification: %s" % repr(msg))
 
     def checkControls(self, controls):
         if controls is not None:
@@ -78,10 +78,10 @@ class BaseLDAPServer(protocol.Protocol):
 
     def handleUnknown(self, request, controls, callback):
         log.msg('Unknown request: %r' % request)
-	msg = pureldap.LDAPExtendedResponse(resultCode=ldaperrors.LDAPProtocolError.resultCode,
+        msg = pureldap.LDAPExtendedResponse(resultCode=ldaperrors.LDAPProtocolError.resultCode,
                                             responseName='1.3.6.1.4.1.1466.20036',
                                             errorMessage='Unknown request')
-	return msg
+        return msg
 
     def _cbLDAPError(self, reason, name):
         reason.trap(ldaperrors.LDAPException)
@@ -108,20 +108,19 @@ class BaseLDAPServer(protocol.Protocol):
                                       errorMessage=reason.getErrorMessage())
 
     def handle(self, msg):
-	assert isinstance(msg.value, pureldap.LDAPProtocolRequest)
+        assert isinstance(msg.value, pureldap.LDAPProtocolRequest)
         if self.debug:
-            log.debug('S<-C %s' % repr(msg))
+            log.msg('S<-C %s' % repr(msg), debug=True)
 
-	if msg.id==0:
-	    self.unsolicitedNotification(msg.value)
-	else:
+        if msg.id==0:
+            self.unsolicitedNotification(msg.value)
+        else:
             name = msg.value.__class__.__name__
             handler = getattr(self, 'handle_'+name, self.handleUnknown)
             d = defer.maybeDeferred(handler,
                                     msg.value,
                                     msg.controls,
                                     lambda response: self._cbHandle(response, msg.id))
-            assert isinstance(d, defer.Deferred)
             d.addErrback(self._cbLDAPError, name)
             d.addErrback(defer.logError)
             d.addErrback(self._cbOtherError, name)
@@ -161,6 +160,7 @@ class LDAPServer(BaseLDAPServer):
 
                 d = entry.bind(auth)
                 def _cb(entry):
+                    self.boundUser=entry
                     msg = pureldap.LDAPBindResponse(
                         resultCode=ldaperrors.Success.resultCode,
                         matchedDN=str(entry.dn))
@@ -183,7 +183,7 @@ class LDAPServer(BaseLDAPServer):
             attributes=[ ('supportedLDAPVersion', ['3']),
                          ('namingContexts', [str(root.dn)]),
                          ('supportedExtension', [
-            pureldap.LDAPPasswordModifyRequest.oid.value,
+            pureldap.LDAPPasswordModifyRequest.oid,
             ]),
                          ],
             ))
@@ -252,6 +252,160 @@ class LDAPServer(BaseLDAPServer):
         d.addCallback(_report)
         return d
 
+    fail_LDAPAddRequest = pureldap.LDAPAddResponse
+
+    def handle_LDAPAddRequest(self, request, controls, reply):
+        self.checkControls(controls)
+
+        attributes = {}
+        for name, vals in request.attributes:
+            attributes.setdefault(name.value, set())
+            attributes[name.value].update([x.value for x in vals])
+        dn = distinguishedname.DistinguishedName(request.entry)
+        rdn = str(dn.split()[0])
+        parent = dn.up()
+        root = interfaces.IConnectedLDAPEntry(self.factory)
+        d = root.lookup(parent)
+        def _gotEntry(parent):
+            d = parent.addChild(rdn, attributes)
+            return d
+        d.addCallback(_gotEntry)
+        def _report(entry):
+            return pureldap.LDAPAddResponse(resultCode=0)
+        d.addCallback(_report)
+        return d
+
+    fail_LDAPModifyDNRequest = pureldap.LDAPModifyDNResponse
+
+    def handle_LDAPModifyDNRequest(self, request, controls, reply):
+        self.checkControls(controls)
+
+        dn = distinguishedname.DistinguishedName(request.entry)
+        newrdn = distinguishedname.RelativeDistinguishedName(request.newrdn)
+        deleteoldrdn = bool(request.deleteoldrdn)
+        if not deleteoldrdn:
+            #TODO support this
+            raise ldaperrors.LDAPUnwillingToPerform("Cannot handle preserving old RDN yet.")
+        newSuperior = request.newSuperior
+        if newSuperior is None:
+            newSuperior = dn.up()
+        else:
+            newSuperior = distinguishedname.DistinguishedName(newSuperior)
+        newdn = distinguishedname.DistinguishedName(
+            listOfRDNs=(newrdn,)+newSuperior.split())
+
+        #TODO make this more atomic
+        root = interfaces.IConnectedLDAPEntry(self.factory)
+        d = root.lookup(dn)
+        def _gotEntry(entry):
+            d = entry.move(newdn)
+            return d
+        d.addCallback(_gotEntry)
+        def _report(entry):
+            return pureldap.LDAPModifyDNResponse(resultCode=0)
+        d.addCallback(_report)
+        return d
+
+    fail_LDAPModifyRequest = pureldap.LDAPModifyResponse
+
+    def handle_LDAPModifyRequest(self, request, controls, reply):
+        self.checkControls(controls)
+
+        root = interfaces.IConnectedLDAPEntry(self.factory)
+        mod = delta.ModifyOp.fromLDAP(request)
+        d = mod.patch(root)
+        def _patched(entry):
+            return entry.commit()
+        d.addCallback(_patched)
+        def _report(entry):
+            return pureldap.LDAPModifyResponse(resultCode=0)
+        d.addCallback(_report)
+        return d
+
+    fail_LDAPExtendedRequest = pureldap.LDAPExtendedResponse
+
+    def handle_LDAPExtendedRequest(self, request, controls, reply):
+        self.checkControls(controls)
+
+        for handler in [getattr(self, attr)
+                        for attr in dir(self)
+                        if attr.startswith('extendedRequest_')]:
+            if getattr(handler, 'oid', None) == request.requestName:
+                berdecoder = getattr(handler, 'berdecoder', None)
+
+                if berdecoder is None:
+                    values = [request.requestValue]
+                else:
+                    values = pureber.berDecodeMultiple(request.requestValue, berdecoder)
+
+                d = defer.maybeDeferred(handler, *values, **{'reply': reply})
+                def eb(fail, oid):
+                    fail.trap(ldaperrors.LDAPException)
+                    return pureldap.LDAPExtendedResponse(
+                        resultCode=fail.value.resultCode,
+                        errorMessage=fail.value.message,
+                        responseName=oid,
+                        )
+                d.addErrback(eb, request.requestName)
+                return d
+
+        raise ldaperrors.LDAPProtocolError('Unknown extended request: %s' % request.requestName)
+
+    def extendedRequest_LDAPPasswordModifyRequest(self, data, reply):
+        if not isinstance(data, pureber.BERSequence):
+            raise ldaperrors.LDAPProtocolError('Extended request PasswordModify expected a BERSequence.')
+
+        userIdentity = None
+        oldPasswd = None
+        newPasswd = None
+
+        for value in data:
+            if isinstance(value, pureldap.LDAPPasswordModifyRequest_userIdentity):
+                if userIdentity is not None:
+                    raise ldaperrors.LDAPProtocolError(
+                        'Extended request PasswordModify received userIdentity twice.')
+                userIdentity = value.value
+            elif isinstance(value, pureldap.LDAPPasswordModifyRequest_oldPasswd):
+                if oldPasswd is not None:
+                    raise ldaperrors.LDAPProtocolError('Extended request PasswordModify received oldPasswd twice.')
+                oldPasswd = value.value
+            elif isinstance(value, pureldap.LDAPPasswordModifyRequest_newPasswd):
+                if newPasswd is not None:
+                    raise ldaperrors.LDAPProtocolError('Extended request PasswordModify received newPasswd twice.')
+                newPasswd = value.value
+            else:
+                raise ldaperrors.LDAPProtocolError('Extended request PasswordModify received unexpected item.')
+
+        if self.boundUser is None:
+            raise ldaperrors.LDAPStrongAuthRequired()
+
+        if (userIdentity is not None
+            and userIdentity != self.boundUser.dn):
+            #TODO this hardcodes ACL
+            log.msg('User %(actor)s tried to change password of %(target)s' % {
+                'actor': str(self.boundUser.dn),
+                'target': str(userIdentity),
+                })
+            raise ldaperrors.LDAPInsufficientAccessRights()
+
+        if (oldPasswd is not None
+            or newPasswd is None):
+            raise ldaperrors.LDAPOperationsError('Password does not support this case.')
+
+        self.boundUser.setPassword(newPasswd)
+        return pureldap.LDAPExtendedResponse(resultCode=ldaperrors.Success.resultCode,
+                                             responseName=self.extendedRequest_LDAPPasswordModifyRequest.oid)
+
+        # TODO
+        if userIdentity is None:
+            userIdentity = str(self.boundUser.dn)
+
+        raise NotImplementedError('VALUE %r' % value)
+    extendedRequest_LDAPPasswordModifyRequest.oid = pureldap.LDAPPasswordModifyRequest.oid
+    extendedRequest_LDAPPasswordModifyRequest.berdecoder = (
+        pureber.BERDecoderContext(
+        inherit=pureldap.LDAPBERDecoderContext_LDAPPasswordModifyRequest(inherit=pureber.BERDecoderContext())))
+
 if __name__ == '__main__':
     """
     Demonstration LDAP server; reads LDIF from stdin and
@@ -262,21 +416,21 @@ if __name__ == '__main__':
     log.startLogging(sys.stderr)
 
     from twisted.python import components
-    from twisted.trial import util
     from ldaptor import inmemory
 
-    d = inmemory.fromLDIFFile(sys.stdin)
-    db = util.deferredResult(d)
-
     class LDAPServerFactory(protocol.ServerFactory):
         def __init__(self, root):
             self.root = root
-
     components.registerAdapter(lambda x: x.root,
                                LDAPServerFactory,
                                interfaces.IConnectedLDAPEntry)
 
-    factory = LDAPServerFactory(db)
-    factory.protocol = LDAPServer
-    reactor.listenTCP(10389, factory)
+    def start(db):
+        factory = LDAPServerFactory(db)
+        factory.protocol = LDAPServer
+        reactor.listenTCP(10389, factory)
+
+    d = inmemory.fromLDIFFile(sys.stdin)
+    d.addCallback(start)
+    d.addErrback(log.err)
     reactor.run()
