@@ -1,5 +1,4 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
+# Copyright (C) 2001 Tommi Virtanen
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -43,17 +42,17 @@ import string
 # |
 # class
 
-CLASS_MASK		= 0xc0
-CLASS_UNIVERSAL	= 0x00
-CLASS_APPLICATION	= 0x40
-CLASS_CONTEXT		= 0x80
-CLASS_PRIVATE		= 0xc0
+CLASS_MASK              = 0xc0
+CLASS_UNIVERSAL = 0x00
+CLASS_APPLICATION       = 0x40
+CLASS_CONTEXT           = 0x80
+CLASS_PRIVATE           = 0xc0
 
-STRUCTURED_MASK		= 0x20
-STRUCTURED		= 0x20
-NOT_STRUCTURED		= 0x00
+STRUCTURED_MASK         = 0x20
+STRUCTURED              = 0x20
+NOT_STRUCTURED          = 0x00
 
-TAG_MASK		= 0x1f
+TAG_MASK                = 0x1f
 
 # LENGTH
 # 0xxxxxxx = 0..127
@@ -80,9 +79,9 @@ def berDecodeLength(m, offset=0):
     l=ber2int(m[offset+0])
     ll=1
     if l&0x80:
-	ll=1+(l&0x7F)
-	need(m, offset+ll)
-	l=ber2int(m[offset+1:offset+ll], signed=0)
+        ll=1+(l&0x7F)
+        need(m, offset+ll)
+        l=ber2int(m[offset+1:offset+ll], signed=0)
     return (l, ll)
 
 def int2berlen(i):
@@ -100,8 +99,8 @@ def int2ber(i, signed=True):
     encoded=''
     while ((signed and (i>127 or i<-128))
            or (not signed and (i>255))):
-	encoded=chr(i%256)+encoded
-	i=i>>8
+        encoded=chr(i%256)+encoded
+        i=i>>8
     encoded=chr(i%256)+encoded
     return encoded
 
@@ -109,45 +108,45 @@ def ber2int(e, signed=True):
     need(e, 1)
     v=0L+ord(e[0])
     if v&0x80 and signed:
-	v=v-256
+        v=v-256
     for i in range(1, len(e)):
-	v=(v<<8) | ord(e[i])
+        v=(v<<8) | ord(e[i])
     return v
 
 class BERBase:
     tag = None
 
     def identification(self):
-	return self.tag
+        return self.tag
 
     def __init__(self, tag=None):
-	if tag is not None:
-	    self.tag=tag
+        if tag is not None:
+            self.tag=tag
 
     def __len__(self):
-	return len(str(self))
+        return len(str(self))
 
     def __cmp__(self, other):
-	if isinstance(other, BERBase):
-	    return cmp(str(self), str(other))
-	else:
-	    return -1
+        if isinstance(other, BERBase):
+            return cmp(str(self), str(other))
+        else:
+            return -1
 
     def __eq__(self, other):
-	if isinstance(other, BERBase):
-	    return str(self) == str(other)
-	else:
-	    return False
+        if isinstance(other, BERBase):
+            return str(self) == str(other)
+        else:
+            return False
 
     def __ne__(self, other):
-	if isinstance(other, BERBase):
-	    return str(self) != str(other)
-	else:
-	    return False
+        if isinstance(other, BERBase):
+            return str(self) != str(other)
+        else:
+            return False
 
 class BERStructured(BERBase):
     def identification(self):
-	return STRUCTURED|self.tag
+        return STRUCTURED|self.tag
 
 class BERException(Exception): pass
 
@@ -156,39 +155,39 @@ class BERExceptionInsufficientData(Exception): pass
 def need(buf, n):
     d=n-len(buf)
     if d>0:
-	raise BERExceptionInsufficientData, d
+        raise BERExceptionInsufficientData, d
 
 class BERInteger(BERBase):
     tag = 0x02
     value = None
 
     def fromBER(klass, tag, content, berdecoder=None):
-	assert len(content)>0
-	value=ber2int(content)
+        assert len(content)>0
+        value=ber2int(content)
         r = klass(value=value, tag=tag)
         return r
     fromBER = classmethod(fromBER)
 
     def __init__(self, value=None, tag=None):
-	"""Create a new BERInteger object.
-	value is an integer.
-	"""
-	BERBase.__init__(self, tag)
-	assert value is not None
+        """Create a new BERInteger object.
+        value is an integer.
+        """
+        BERBase.__init__(self, tag)
+        assert value is not None
         self.value=value
 
     def __str__(self):
-	encoded=int2ber(self.value)
-	return chr(self.identification()) \
-	       +int2berlen(len(encoded)) \
-	       +encoded
+        encoded=int2ber(self.value)
+        return chr(self.identification()) \
+               +int2berlen(len(encoded)) \
+               +encoded
 
     def __repr__(self):
-	if self.tag==self.__class__.tag:
-	    return self.__class__.__name__+"(value=%r)"%self.value
-	else:
-	    return self.__class__.__name__+"(value=%r, tag=%d)" \
-		   %(self.value, self.tag)
+        if self.tag==self.__class__.tag:
+            return self.__class__.__name__+"(value=%r)"%self.value
+        else:
+            return self.__class__.__name__+"(value=%r, tag=%d)" \
+                   %(self.value, self.tag)
 
 class BEROctetString(BERBase):
     tag = 0x04
@@ -196,83 +195,84 @@ class BEROctetString(BERBase):
     value = None
 
     def fromBER(klass, tag, content, berdecoder=None):
-	assert len(content)>=0
+        assert len(content)>=0
         r = klass(value=content, tag=tag)
         return r
     fromBER = classmethod(fromBER)
 
     def __init__(self, value=None, tag=None):
-	BERBase.__init__(self, tag)
-	assert value is not None
+        BERBase.__init__(self, tag)
+        assert value is not None
         self.value=value
 
     def __str__(self):
-	return chr(self.identification()) \
-	       +int2berlen(len(self.value)) \
-	       +self.value
+        value = str(self.value)
+        return chr(self.identification()) \
+               +int2berlen(len(value)) \
+               +value
 
     def __repr__(self):
-	if self.tag==self.__class__.tag:
-	    return self.__class__.__name__+"(value=%s)" \
-		   %repr(self.value)
-	else:
-	    return self.__class__.__name__ \
-		   +"(value=%s, tag=%d)" \
-		   %(repr(self.value), self.tag)
+        if self.tag==self.__class__.tag:
+            return self.__class__.__name__+"(value=%s)" \
+                   %repr(self.value)
+        else:
+            return self.__class__.__name__ \
+                   +"(value=%s, tag=%d)" \
+                   %(repr(self.value), self.tag)
 
 class BERNull(BERBase):
     tag = 0x05
 
     def fromBER(klass, tag, content, berdecoder=None):
-	assert len(content) == 0
+        assert len(content) == 0
         r = klass(tag=tag)
         return r
     fromBER = classmethod(fromBER)
 
     def __init__(self, tag=None):
-	BERBase.__init__(self, tag)
+        BERBase.__init__(self, tag)
 
     def __str__(self):
-	return chr(self.identification())+chr(0)
+        return chr(self.identification())+chr(0)
 
     def __repr__(self):
-	if self.tag==self.__class__.tag:
-	    return self.__class__.__name__+"()"
-	else:
-	    return self.__class__.__name__+"(tag=%d)"%self.tag
+        if self.tag==self.__class__.tag:
+            return self.__class__.__name__+"()"
+        else:
+            return self.__class__.__name__+"(tag=%d)"%self.tag
 
 class BERBoolean(BERBase):
     tag = 0x01
 
     def fromBER(klass, tag, content, berdecoder=None):
-	assert len(content) > 0
-	value = ber2int(content)
+        assert len(content) > 0
+        value = ber2int(content)
         r = klass(value=value, tag=tag)
         return r
     fromBER = classmethod(fromBER)
 
     def __init__(self, value=None, tag=None):
-	"""Create a new BERInteger object.
-	value is an integer.
-	"""
-	BERBase.__init__(self, tag)
-	assert value is not None
+        """Create a new BERInteger object.
+        value is an integer.
+        """
+        BERBase.__init__(self, tag)
+        assert value is not None
         if value:
             value=0xFF
         self.value=value
 
     def __str__(self):
-	assert self.value==0 or self.value==0xFF
-	return chr(self.identification()) \
-	       +int2berlen(1) \
-	       +chr(self.value)
+        assert self.value==0 or self.value==0xFF
+        return chr(self.identification()) \
+               +int2berlen(1) \
+               +chr(self.value)
 
     def __repr__(self):
-	if self.tag==self.__class__.tag:
-	    return self.__class__.__name__+"(value=%d)"%self.value
-	else:
-	    return self.__class__.__name__+"(value=%d, tag=%d)" \
-		   %(self.value, self.tag)
+        if self.tag==self.__class__.tag:
+            return self.__class__.__name__+"(value=%d)"%self.value
+        else:
+            return self.__class__.__name__+"(value=%d, tag=%d)" \
+                   %(self.value, self.tag)
 
 
 class BEREnumerated(BERInteger):
@@ -289,21 +289,21 @@ class BERSequence(BERStructured, UserList.UserList):
     fromBER = classmethod(fromBER)
 
     def __init__(self, value=None, tag=None):
-	BERStructured.__init__(self, tag)
-	UserList.UserList.__init__(self)
-	assert value is not None
+        BERStructured.__init__(self, tag)
+        UserList.UserList.__init__(self)
+        assert value is not None
         self[:]=value
 
     def __str__(self):
-	r=string.join(map(str, self.data), '')
-	return chr(self.identification())+int2berlen(len(r))+r
+        r=string.join(map(str, self.data), '')
+        return chr(self.identification())+int2berlen(len(r))+r
 
     def __repr__(self):
-	if self.tag==self.__class__.tag:
-	    return self.__class__.__name__+"(value=%s)"%repr(self.data)
-	else:
-	    return self.__class__.__name__+"(value=%s, tag=%d)" \
-		   %(repr(self.data), self.tag)
+        if self.tag==self.__class__.tag:
+            return self.__class__.__name__+"(value=%s)"%repr(self.data)
+        else:
+            return self.__class__.__name__+"(value=%s, tag=%d)" \
+                   %(repr(self.data), self.tag)
 
 
 class BERSequenceOf(BERSequence):
@@ -317,63 +317,63 @@ class BERSet(BERSequence):
 
 class BERDecoderContext:
     Identities = {
-	BERInteger.tag: BERInteger,
-	BEROctetString.tag: BEROctetString,
-	BERNull.tag: BERNull,
-	BERBoolean.tag: BERBoolean,
-	BEREnumerated.tag: BEREnumerated,
-	BERSequence.tag: BERSequence,
-	BERSet.tag: BERSet,
-	}
+        BERInteger.tag: BERInteger,
+        BEROctetString.tag: BEROctetString,
+        BERNull.tag: BERNull,
+        BERBoolean.tag: BERBoolean,
+        BEREnumerated.tag: BEREnumerated,
+        BERSequence.tag: BERSequence,
+        BERSet.tag: BERSet,
+        }
 
     def __init__(self, fallback=None, inherit=None):
-	self.fallback=fallback
-	self.inherit_context=inherit
+        self.fallback=fallback
+        self.inherit_context=inherit
 
     def lookup_id(self, id):
-	try:
-	    return self.Identities[id]
-	except KeyError:
-	    if self.fallback:
-		return self.fallback.lookup_id(id)
-	    else:
-		return None
+        try:
+            return self.Identities[id]
+        except KeyError:
+            if self.fallback:
+                return self.fallback.lookup_id(id)
+            else:
+                return None
 
     def inherit(self):
-	return self.inherit_context or self
+        return self.inherit_context or self
 
     def __repr__(self):
         identities = []
         for tag, class_ in self.Identities.items():
             identities.append('0x%02x: %s' % (tag, class_.__name__))
-	return "<"+self.__class__.__name__ \
-	       +" identities={%s}" % ', '.join(identities) \
-	       +" fallback="+repr(self.fallback) \
-	       +" inherit="+repr(self.inherit_context) \
-	       +">"
+        return "<"+self.__class__.__name__ \
+               +" identities={%s}" % ', '.join(identities) \
+               +" fallback="+repr(self.fallback) \
+               +" inherit="+repr(self.inherit_context) \
+               +">"
 
 def berDecodeObject(context, m):
     """berDecodeObject(context, string) -> (berobject, bytesUsed)
     berobject may be None.
     """
     while m:
-	need(m, 2)
-	i=ber2int(m[0], signed=0)&(CLASS_MASK|TAG_MASK)
+        need(m, 2)
+        i=ber2int(m[0], signed=0)&(CLASS_MASK|TAG_MASK)
 
         length, lenlen = berDecodeLength(m, offset=1)
         need(m, 1+lenlen+length)
         m2 = m[1+lenlen:1+lenlen+length]
 
-	berclass=context.lookup_id(i)
-	if berclass:
-	    inh=context.inherit()
-	    assert inh
+        berclass=context.lookup_id(i)
+        if berclass:
+            inh=context.inherit()
+            assert inh
             r = berclass.fromBER(tag=i,
                                  content=m2,
                                  berdecoder=inh)
             return (r, 1+lenlen+length)
-	else:
-	    #raise UnknownBERTag, (i, context)
+        else:
+            #raise UnknownBERTag, (i, context)
             print str(UnknownBERTag(i, context)) #TODO
             return (None, 1+lenlen+length)
     return (None, 0)
