@@ -1,5 +1,4 @@
-# Ldaptor -- TODO
-# Copyright (C) 2001 Matthew W. Lefkowitz
+# Copyright (C) 2001 Tommi Virtanen
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -26,9 +25,9 @@ def s(*l):
     """Join all members of list to a string. Integer members are chr()ed"""
     r=''
     for e in l:
-	if isinstance(e, types.IntType):
-	    e=chr(e)
-	r=r+str(e)
+        if isinstance(e, types.IntType):
+            e=chr(e)
+        r=r+str(e)
     return r
 
 def l(s):
@@ -38,118 +37,144 @@ def l(s):
 class KnownValues(unittest.TestCase):
     knownValues=( # class, args, kwargs, expected_result
 
-	(pureldap.LDAPModification_delete,
-	 [],
-         { "attributeType": 'bar',
-	   },
+        (pureldap.LDAPModifyRequest,
+         [],
+         { "object": 'cn=foo, dc=example, dc=com',
+           "modification": [
+                      pureber.BERSequence([
+                        pureber.BEREnumerated(0),
+                        pureber.BERSequence([
+                          pureldap.LDAPAttributeDescription('bar'),
+                          pureber.BERSet([
+                            pureldap.LDAPString('a'),
+                            pureldap.LDAPString('b'),
+                            ]),
+                          ]),
+                        ]),
+                      ],
+           },
          None,
-	 [0x30, 0x0c]
-	 + [0x0a, 0x01, 0x01]
-         + [0x30, 0x07]
-	 + [0x04, 0x03] + l("bar")
-	 + [0x31, 0x00]),
+         [0x66, 50]
+         + ([0x04, 0x1a] + l("cn=foo, dc=example, dc=com")
+            + [0x30, 20]
+            + ([0x30, 18]
+               + ([0x0a, 0x01, 0x00]
+                  + [0x30, 13]
+                  + ([0x04, len("bar")] + l("bar")
+                     + [0x31, 0x06]
+                     + ([0x04, len("a")] + l("a")
+                        + [0x04, len("b")] + l("b"))))))
+            ),
 
-	(pureldap.LDAPModifyRequest,
-	 [],
-	 { "object": 'cn=foo, dc=example, dc=com',
-	   "modification": [pureldap.LDAPModification_delete('bar')]
-	   },
+        (pureldap.LDAPModifyRequest,
+         [],
+         { "object": 'cn=foo, dc=example, dc=com',
+           "modification": [
+                      pureber.BERSequence([
+                        pureber.BEREnumerated(1L),
+                        pureber.BERSequence([
+                          pureber.BEROctetString('bar'),
+                          pureber.BERSet([]),
+                          ]),
+                        ]),
+                      ],
+           },
          None,
-	 [0x66, 0x2c]
-	 + [0x04, 0x1a]
-	 + l("cn=foo, dc=example, dc=com")
-	 + [0x30, 0x0e]
-	 + [0x30, 0x0c]
-	 + [0x0a, 0x01, 0x01]
-	 + [0x30, 0x07]
-	 + [0x04, 0x03] + l("bar")
-	 + [0x31, 0x00]),
+         [0x66, 0x2c]
+         + ([0x04, 0x1a] + l("cn=foo, dc=example, dc=com")
+            + [0x30, 0x0e]
+            + ([0x30, 0x0c]
+               + ([0x0a, 0x01, 0x01]
+                  + [0x30, 0x07]
+                  + ([0x04, 0x03] + l("bar")
+                     + [0x31, 0x00]))))
+        ),
 
-	(pureldap.LDAPFilter_not,
-	 [],
-	 { "value": pureldap.LDAPFilter_present("foo"),
-	   },
+        (pureldap.LDAPFilter_not,
+         [],
+         { "value": pureldap.LDAPFilter_present("foo"),
+           },
          pureldap.LDAPBERDecoderContext_Filter(fallback=pureber.BERDecoderContext()),
-	 [0xa2, 0x05]
-	 + [0x87]
-	 + [len("foo")]
-	 + l("foo")),
+         [0xa2, 0x05]
+         + [0x87]
+         + [len("foo")]
+         + l("foo")),
 
-	(pureldap.LDAPFilter_or,
-	 [],
-	 { "value": [pureldap.LDAPFilter_equalityMatch(
+        (pureldap.LDAPFilter_or,
+         [],
+         { "value": [pureldap.LDAPFilter_equalityMatch(
         attributeDesc=pureldap.LDAPAttributeDescription(value='cn'),
         assertionValue=pureldap.LDAPAssertionValue(value='foo')),
                      pureldap.LDAPFilter_equalityMatch(
         attributeDesc=pureldap.LDAPAttributeDescription(value='uid'),
         assertionValue=pureldap.LDAPAssertionValue(value='foo')),
                      ]
-	   },
+           },
          pureldap.LDAPBERDecoderContext_Filter(fallback=pureber.BERDecoderContext()),
-	 [0xa1, 23]
-	 + [0xa3, 9]
-	 + [0x04] + [len("cn")] + l("cn")
-	 + [0x04] + [len("foo")] + l("foo")
-	 + [0xa3, 10]
-	 + [0x04] + [len("uid")] + l("uid")
-	 + [0x04] + [len("foo")] + l("foo"),
+         [0xa1, 23]
+         + [0xa3, 9]
+         + [0x04] + [len("cn")] + l("cn")
+         + [0x04] + [len("foo")] + l("foo")
+         + [0xa3, 10]
+         + [0x04] + [len("uid")] + l("uid")
+         + [0x04] + [len("foo")] + l("foo"),
          ),
 
-	(pureldap.LDAPFilter_and,
-	 [],
-	 { "value": [pureldap.LDAPFilter_equalityMatch(
+        (pureldap.LDAPFilter_and,
+         [],
+         { "value": [pureldap.LDAPFilter_equalityMatch(
         attributeDesc=pureldap.LDAPAttributeDescription(value='cn'),
         assertionValue=pureldap.LDAPAssertionValue(value='foo')),
                      pureldap.LDAPFilter_equalityMatch(
         attributeDesc=pureldap.LDAPAttributeDescription(value='uid'),
         assertionValue=pureldap.LDAPAssertionValue(value='foo')),
                      ]
-	   },
+           },
          pureldap.LDAPBERDecoderContext_Filter(fallback=pureber.BERDecoderContext()),
-	 [0xa0, 23]
-	 + [0xa3, 9]
-	 + [0x04] + [len("cn")] + l("cn")
-	 + [0x04] + [len("foo")] + l("foo")
-	 + [0xa3, 10]
-	 + [0x04] + [len("uid")] + l("uid")
-	 + [0x04] + [len("foo")] + l("foo"),
+         [0xa0, 23]
+         + [0xa3, 9]
+         + [0x04] + [len("cn")] + l("cn")
+         + [0x04] + [len("foo")] + l("foo")
+         + [0xa3, 10]
+         + [0x04] + [len("uid")] + l("uid")
+         + [0x04] + [len("foo")] + l("foo"),
          ),
 
-	(pureldap.LDAPModifyDNRequest,
-	 [],
-	 {'entry': 'cn=foo,dc=example,dc=com',
-	  'newrdn': 'uid=bar',
-	  'deleteoldrdn': 0,
-	  },
+        (pureldap.LDAPModifyDNRequest,
+         [],
+         {'entry': 'cn=foo,dc=example,dc=com',
+          'newrdn': 'uid=bar',
+          'deleteoldrdn': 0,
+          },
          None,
-	 [0x6c, 0x26]
-	 + [0x04]
-	 + [len("cn=foo,dc=example,dc=com")]
-	 + l("cn=foo,dc=example,dc=com")
-	 + [0x04]
-	 + [len("uid=bar")]
-	 + l("uid=bar")
-	 + [0x01, 0x01, 0x00]),
+         [0x6c, 0x26]
+         + [0x04]
+         + [len("cn=foo,dc=example,dc=com")]
+         + l("cn=foo,dc=example,dc=com")
+         + [0x04]
+         + [len("uid=bar")]
+         + l("uid=bar")
+         + [0x01, 0x01, 0x00]),
 
-	(pureldap.LDAPModifyDNRequest,
-	 [],
-	 {'entry': 'cn=aoue,dc=example,dc=com',
-	  'newrdn': 'uid=aoue',
-	  'deleteoldrdn': 0,
-	  'newSuperior': 'ou=People,dc=example,dc=com',
-	  },
+        (pureldap.LDAPModifyDNRequest,
+         [],
+         {'entry': 'cn=aoue,dc=example,dc=com',
+          'newrdn': 'uid=aoue',
+          'deleteoldrdn': 0,
+          'newSuperior': 'ou=People,dc=example,dc=com',
+          },
          None,
-	 [0x6c, 69]
-	 + [0x04]
-	 + [len("cn=aoue,dc=example,dc=com")]
-	 + l("cn=aoue,dc=example,dc=com")
-	 + [0x04]
-	 + [len("uid=aoue")]
-	 + l("uid=aoue")
-	 + [0x01, 0x01, 0x00]
-	 + [0x80]
-	 + [len("ou=People,dc=example,dc=com")]
-	 + l("ou=People,dc=example,dc=com")),
+         [0x6c, 69]
+         + [0x04]
+         + [len("cn=aoue,dc=example,dc=com")]
+         + l("cn=aoue,dc=example,dc=com")
+         + [0x04]
+         + [len("uid=aoue")]
+         + l("uid=aoue")
+         + [0x01, 0x01, 0x00]
+         + [0x80]
+         + [len("ou=People,dc=example,dc=com")]
+         + l("ou=People,dc=example,dc=com")),
 
         (pureldap.LDAPSearchRequest,
          [],
@@ -181,7 +206,7 @@ class KnownValues(unittest.TestCase):
          {},
          None,
          [0x42, 0x00]
-	),
+        ),
 
         (pureldap.LDAPSearchResultDone,
          [],
@@ -201,7 +226,7 @@ class KnownValues(unittest.TestCase):
          + l('')
          # referral, TODO
          + []
-	),
+        ),
 
         (pureldap.LDAPSearchResultDone,
          [],
@@ -222,7 +247,7 @@ class KnownValues(unittest.TestCase):
          + l('')
          # referral, TODO
          + []
-	),
+        ),
 
         (pureldap.LDAPSearchResultDone,
          [],
@@ -244,7 +269,7 @@ class KnownValues(unittest.TestCase):
          + l('the foobar was fubar',)
          # referral, TODO
          + []
-	),
+        ),
 
         (pureldap.LDAPSearchResultDone,
          [],
@@ -265,7 +290,7 @@ class KnownValues(unittest.TestCase):
          + l('the foobar was fubar',)
          # referral, TODO
          + []
-	),
+        ),
 
         (pureldap.LDAPMessage,
          [],
@@ -505,48 +530,91 @@ class KnownValues(unittest.TestCase):
             )
          ),
 
+        (pureldap.LDAPExtendedRequest,
+         [],
+         {'requestName': '42.42.42',
+          'requestValue': 'foo',
+          },
+         None,
+         [0x40|0x20|23, 1+1+8+1+1+3]
+         + ([0x80|0]
+            + [len('42.42.42')]
+            + l('42.42.42'))
+         + ([0x80|1]
+            + [len('foo')]
+            + l('foo'))
+         ),
+
+        (pureldap.LDAPExtendedRequest,
+         [],
+         {'requestName': '42.42.42',
+          'requestValue': None,
+          },
+         None,
+         [0x40|0x20|23, 1+1+8]
+         + ([0x80|0]
+            + [len('42.42.42')]
+            + l('42.42.42'))
+         ),
+
+        (pureldap.LDAPAbandonRequest,
+         [],
+         {'id': 3},
+         None,
+         [0x40|0x10, 0x01, 3]
+         ),
+
+        (pureldap.LDAPBindRequest,
+         [],
+         {'auth': ('PLAIN', 'test'),
+          'sasl': True},
+         pureldap.LDAPBERDecoderContext(
+                fallback=pureldap.LDAPBERDecoderContext(fallback=pureber.BERDecoderContext()),
+                inherit=pureldap.LDAPBERDecoderContext(fallback=pureber.BERDecoderContext())),
+         [ord(x) for x in str(pureldap.LDAPBindRequest(auth=('PLAIN', 'test'), sasl=True))]
+         )
         )
 
     def testToLDAP(self):
-	"""str(LDAPClass(...)) should give known result with known input"""
-	for klass, args, kwargs, decoder, encoded in self.knownValues:
-	    result = klass(*args, **kwargs)
-	    result = str(result)
-	    result = map(ord, result)
-	    if result!=encoded:
-		raise AssertionError, \
-		      "Class %s(*%s, **%s) doesn't encode properly: " \
-		      "%s != %s" % (klass.__name__,
-				    repr(args), repr(kwargs),
-				    repr(result), repr(encoded))
+        """str(LDAPClass(...)) should give known result with known input"""
+        for klass, args, kwargs, decoder, encoded in self.knownValues:
+            result = klass(*args, **kwargs)
+            result = str(result)
+            result = map(ord, result)
+            if result!=encoded:
+                raise AssertionError, \
+                      "Class %s(*%s, **%s) doesn't encode properly: " \
+                      "%s != %s" % (klass.__name__,
+                                    repr(args), repr(kwargs),
+                                    repr(result), repr(encoded))
 
     def testFromLDAP(self):
-	"""LDAPClass(encoded="...") should give known result with known input"""
-	for klass, args, kwargs, decoder, encoded in self.knownValues:
+        """LDAPClass(encoded="...") should give known result with known input"""
+        for klass, args, kwargs, decoder, encoded in self.knownValues:
             if decoder is None:
                 decoder = pureldap.LDAPBERDecoderContext(
                     fallback=pureber.BERDecoderContext())
-	    m=s(*encoded)
+            m=s(*encoded)
             result, bytes = pureber.berDecodeObject(decoder, m)
-	    self.assertEquals(bytes, len(m))
+            self.assertEquals(bytes, len(m))
 
-	    shouldBe = klass(*args, **kwargs)
-	    #TODO shouldn't use str below
-	    assert str(result)==str(shouldBe), \
-		   "Class %s(*%s, **%s) doesn't decode properly: " \
-		   "%s != %s" % (klass.__name__,
-				 repr(args), repr(kwargs),
-				 repr(result), repr(shouldBe))
+            shouldBe = klass(*args, **kwargs)
+            #TODO shouldn't use str below
+            assert str(result)==str(shouldBe), \
+                   "Class %s(*%s, **%s) doesn't decode properly: " \
+                   "%s != %s" % (klass.__name__,
+                                 repr(args), repr(kwargs),
+                                 repr(result), repr(shouldBe))
 
     def testPartial(self):
-	"""LDAPClass(encoded="...") with too short input should throw BERExceptionInsufficientData"""
-	for klass, args, kwargs, decoder, encoded in self.knownValues:
+        """LDAPClass(encoded="...") with too short input should throw BERExceptionInsufficientData"""
+        for klass, args, kwargs, decoder, encoded in self.knownValues:
             if decoder is None:
                 decoder = pureldap.LDAPBERDecoderContext(
                     fallback=pureber.BERDecoderContext())
-	    for i in xrange(1, len(encoded)):
-		m=s(*encoded)[:i]
-		self.assertRaises(pureber.BERExceptionInsufficientData,
+            for i in xrange(1, len(encoded)):
+                m=s(*encoded)[:i]
+                self.assertRaises(pureber.BERExceptionInsufficientData,
                                   pureber.berDecodeObject,
                                   decoder, m)
             self.assertEquals((None, 0), pureber.berDecodeObject(decoder, ''))
@@ -561,24 +629,41 @@ class TestEquality(unittest.TestCase):
          [ pureldap.LDAPAttributeDescription(value='cn'),
            pureldap.LDAPAssertionValue(value='bar'),
            ]),
-	(pureber.BERInteger, [0]),
-	)
+        (pureber.BERInteger, [0]),
+        )
 
     def testEquality(self):
-	"""LDAP objects equal LDAP objects with same type and content"""
-	for class_, args in self.valuesToTest:
-	    x=class_(*args)
-	    y=class_(*args)
-	    self.assertEquals(x, x)
-	    self.assertEquals(x, y)
+        """LDAP objects equal LDAP objects with same type and content"""
+        for class_, args in self.valuesToTest:
+            x=class_(*args)
+            y=class_(*args)
+            self.assertEquals(x, x)
+            self.assertEquals(x, y)
 
     def testInEquality(self):
-	"""LDAP objects do not equal LDAP objects with different type or content"""
-	for i in xrange(len(self.valuesToTest)):
-	    for j in xrange(len(self.valuesToTest)):
-		if i!=j:
-		    i_class, i_args = self.valuesToTest[i]
-		    j_class, j_args = self.valuesToTest[j]
-		    x=i_class(*i_args)
-		    y=j_class(*j_args)
-		    self.assertNotEquals(x, y)
+        """LDAP objects do not equal LDAP objects with different type or content"""
+        for i in xrange(len(self.valuesToTest)):
+            for j in xrange(len(self.valuesToTest)):
+                if i!=j:
+                    i_class, i_args = self.valuesToTest[i]
+                    j_class, j_args = self.valuesToTest[j]
+                    x=i_class(*i_args)
+                    y=j_class(*j_args)
+                    self.assertNotEquals(x, y)
+
+class Substrings(unittest.TestCase):
+    def test_length(self):
+        """LDAPFilter_substrings.substrings behaves like a proper list."""
+        decoder = pureldap.LDAPBERDecoderContext(
+            fallback=pureber.BERDecoderContext())
+        filt = pureldap.LDAPFilter_substrings.fromBER(
+            tag=pureldap.LDAPFilter_substrings.tag,
+            content=s(0x04, 4, 'mail',
+                      0x30, 6,
+                      0x80, 4, 'foo@'),
+            berdecoder=decoder)
+        # The confusion that used to occur here was because
+        # filt.substrings was left as a BERSequence, which under the
+        # current str()-to-wire-protocol system had len() > 1 even
+        # when empty, and that tripped e.g. entry.match()
+        self.assertEquals(len(filt.substrings), 1)
