@@ -1,9 +1,14 @@
 """Utilities for writing Twistedy unit tests and debugging."""
 
-from twisted.internet import reactor, defer
+from twisted.internet import defer
+from twisted.python import failure
+from twisted.trial import unittest
 from twisted.test import proto_helpers
 from ldaptor import config
 
+def mustRaise(dummy):
+    raise unittest.FailTest('Should have raised an exception.')
+
 def calltrace():
     """Print out all function calls. For debug use only."""
     def printfuncnames(frame, event, arg):
@@ -32,23 +37,37 @@ class LDAPClientTestDriver:
     messages are stored in self.sent, so you can assert that the sent
     messages are what they are supposed to be.
 
+    It is also possible to include a Failure instance instead of a list
+    of LDAPProtocolResponses which will cause the errback to be called
+    with the failure.
+
     """
     def __init__(self, *responses):
         self.sent=[]
         self.responses=list(responses)
         self.connected = None
         self.transport = FakeTransport(self)
+
     def send(self, op):
         self.sent.append(op)
         l = self._response()
         assert len(l) == 1, \
                "got %d responses for a .send()" % len(l)
-        return defer.succeed(l[0])
+        r = l[0]
+        if isinstance(r, failure.Failure):
+            return defer.fail(r)
+        else:
+            return defer.succeed(r)
+
     def send_multiResponse(self, op, handler, *args, **kwargs):
+        d = defer.Deferred()
         self.sent.append(op)
         responses = self._response()
         while responses:
             r = responses.pop(0)
+            if isinstance(r, failure.Failure):
+                d.errback(r)
+                break
             ret = handler(r, *args, **kwargs)
             if responses:
                 assert not ret, \
@@ -56,6 +75,7 @@ class LDAPClientTestDriver:
             else:
                 assert ret, \
                        "no more responses to give, but handler still wants more (got %r)." % ret
+        return d
 
     def send_noResponse(self, op):
         responses = self.responses.pop(0)
@@ -79,8 +99,8 @@ class LDAPClientTestDriver:
             shouldBeSent,
             self.sent)
         sentStr = ''.join([str(x) for x in self.sent])
-	shouldBeSentStr = ''.join([str(x) for x in shouldBeSent])
-	assert sentStr == shouldBeSentStr, \
+        shouldBeSentStr = ''.join([str(x) for x in shouldBeSent])
+        assert sentStr == shouldBeSentStr, \
                '%s expected to send data %r but sent %r' % (
             self.__class__.__name__,
             shouldBeSentStr,
@@ -108,9 +128,6 @@ def createServer(proto, *responses, **kw):
         #TODO factory.startedConnecting(c)
         proto = factory.buildProtocol(addr=None)
         proto.connectionMade()
-    cfg = config.loadConfig(
-        configFiles=[],
-        reload=True)
     overrides = kw.setdefault('serviceLocationOverrides', {})
     overrides.setdefault('', createClient)
     conf = config.LDAPConfig(**kw)
